class PUBGDataParser:
    def __repr__(self):
        return '<{} {}>'.format(self.__class__.__name__, self.get_id())

    def get_id(self):
        return 'No ID'

    def get_type(self):
        return self.data_type

class Player(PUBGDataParser):
    def __init__(self, raw_data):
        data = raw_data['data'][0]

        self.data_type = data.get('type')
        self.account_id = data.get('id')
        self.attributes = data.get('attributes')
        self.player_name = self.attributes.get('name')

    def get_id(self):
        return self.account_id

class Season(PUBGDataParser):
    def __init__(self, data):
        self.data_type = data['type']
        self.season_id = data['id']
        attributes = data['attributes']
        self.is_current_season = attributes['isCurrentSeason']

    def get_id(self):
        return self.season_id

class Match(PUBGDataParser):
    def __init__(self, data):
        self.data_type = data['data']['type']
        self.match_id = data['data']['id']
        self.assets = list(filter(lambda item: item.get('type') == 'asset', data.get('included')))

    def get_id(self):
        return self.match_id

    def get_telemetry_url(self):
        if len(self.assets) <= 0:
            return None
        asset = self.assets[0]
        return asset['attributes']['URL']

class Seasons:
    def __init__(self, raw_data):
        data = raw_data['data']
        self.seasons = []
        for d in data:
            season = Season(d)
            if season.is_current_season:
                self.current_season = season
            self.seasons.append(season)

    def get_all_seasons(self):
        return self.seasons

    def get_current_season(self):
        return self.current_season

class Matches:
    GAME_MODES = [
            'matchesSolo',
            'matchesDuo',
            'matchesSquad',
            'matchesSoloFPP',
            'matchesDuoFPP',
            'matchesSquadFPP',
            ]
    def __init__(self, raw_data):
        data = raw_data['data']
        self.matches = {}
        relationships = data.get('relationships')

        for mode in self.GAME_MODES:
            self.matches[mode] = []
            matches_data = relationships.get(mode)['data']

            for md in matches_data:
                self.matches[mode].append(md['id'])

    def get_matches(self, game_mode):
        return self.matches.get(game_mode)




class Event:
    def __init__(self, data):
        self.timestamp = data['_D']
        self.data_type = data['_T']
        self.process(data)

    def __repr__(self):
        return '<{} {}>'.format(self.__class__.__name__, self.get_type())

    def get_type(self):
        return self.data_type

    def process(self, data):
        pass

class AttackEvent(Event):
    def process(self, data):
        self.victim = data.get('victim')
        self.damage_reason = data.get('damageReason')
        self.damage_causer_name = data.get('damageCauserName')

class PlayerKillEvent(AttackEvent):
    pass

class PlayerMakeGroggyEvent(AttackEvent):
    pass



class PUBGQuery:
    PUBG_HOST = 'https://api.pubg.com/'

    def get_host(self):
        return self.PUBG_HOST

    def get_domain(self):
        return ''

    def get_endpoint(self):
        return self.get_host() + self.get_domain()

class PUBGShardedQuery(PUBGQuery):
    def __init__(self, shard='steam'):
        self.shard = shard

    def get_host(self):
        return self.PUBG_HOST + 'shards/{}/'.format(self.shard)

class PlayerQuery(PUBGShardedQuery):
    def __init__(self, player_name, shard='steam'):
        self.player_name = player_name
        self.shard = shard

    def get_domain(self):
        return 'players?filter[playerNames]={}'.format(self.player_name)

class SeasonsQuery(PUBGShardedQuery):
    def __init__(self, shard='steam'):
        self.shard = shard

    def get_domain(self):
        return 'seasons'

class MatchesQuery(PUBGShardedQuery):
    def __init__(self, account_id, season_id, shard='steam'):
        self.account_id = account_id
        self.season_id = season_id
        self.shard = shard

    def get_domain(self):
        return 'players/{}/seasons/{}'.format(self.account_id, self.season_id)

class MatchesModeQuery(PUBGShardedQuery):
    """
    Not recommended.
    This query is not that useful as the argument needed for game_mode is not consitent with
    the keys used for data. For example, in solo games,
    game_mode is 'solo' while the key is 'matchesSolo'
    """
    def __init__(self, account_id, season_id, game_mode, shard='steam'):
        self.account_id = account_id
        self.season_id = season_id
        self.game_mode = game_mode
        self.shard = shard

    def get_domain(self):
        return 'seasons/{}/gameMode/{}/players/?filter[playerIds]={}'.format(
            self.season_id,
            self.game_mode,
            self.account_id,
        )

class MatchQuery(PUBGShardedQuery):
    def __init__(self, match_id, shard='steam'):
        self.match_id = match_id
        self.shard = shard

    def get_domain(self):
        return 'matches/{}'.format(self.match_id)

class TelemetryQuery(PUBGQuery):
    def __init__(self, url):
        self.url = url

    def get_endpoint(self):
        return self.url



import setuptools

with open('README.md', 'r') as fh:
    long_description = fh.read()

setuptools.setup(
    name = 'pubg-toolbox',
    version = '0.0.4',
    author = 'Junhan Zhu',
    author_email = 'junhanoct@gmail.com',
    description = 'Simple queries and handles for PUBG data analysis',
    long_description = long_description,
    long_description_content_type = 'text/markdown',
    url = 'https://github.com/junhan-z/pubg-toolbox',
    packages = setuptools.find_packages(),
    classifiers = [
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Topic :: Utilities',
    ],
    python_requires = '>=3',
)



import json
import requests

class PUBGClient:
    BASE_URL = 'https://api.pubg.com/'
    def __init__(self, api_key):
        self.session = requests.Session()
        self.unlimited_session = requests.Session()

        headers = {
                'Accept': 'application/vnd.api+json',
                'Authorization': 'Bearer {}'.format(api_key),
                }

        self.session.headers.update(headers)

    def request(self, query):
        endpoint = query.get_endpoint()
        print('[limited] endpoint: {}'.format(endpoint))
        response = self.session.get(endpoint, timeout = 30)
        if response.status_code != 200:
            print('error: status: {}'.format(response.status_code))
            return ''

        return json.loads(response.text)
